#include "server_data_interact.h"

#include <engine_engine_number_9/baza_throw.h>
#include <engine_engine_number_9/http_structures/cookies.h>

namespace iu9cawebchat {
    const char* stringify_user_chat_role(int64_t role) {
        if (role == user_chat_role_admin)
            return "admin";
        if (role == user_chat_role_regular)
            return "regular";
        if (role == user_chat_role_read_only)
            return "read-only";
        return "not-a-member";
    }

    int64_t find_user_by_credentials (SqliteConnection& conn, const std::string& nickname, const std::string& password) {
        SqliteStatement sql_req(conn,
            "SELECT `id` FROM `user` WHERE `nickname` = ?1 AND `password` = ?2",
            {}, {{1, nickname}, {2, password}});
        fsql_integer_or_null id_col;
        int status = sqlite_stmt_step(sql_req, {{0, &id_col}}, {});
        if (status == SQLITE_ROW) {
            een9_ASSERT_pl(id_col.exist & id_col.value >= 0);
            return id_col.value;
        }
        return -1;
    }

    std::string find_user_name (SqliteConnection& conn, int64_t uid) {
        een9_ASSERT(uid >= 0, "Are you crazy?");
        SqliteStatement sql_req(conn,
           "SELECT `name` FROM `user` WHERE `id` = ?1",
           {{1, uid}}, {});
        fsql_text8_or_null name_col;
        int status = sqlite_stmt_step(sql_req, {}, {{0, &name_col}});
        if (status == SQLITE_ROW) {
            een9_ASSERT_pl(name_col.exist);
            return name_col.value;
        }
        return "";
    }

    RowUser_Content lookup_user_content(SqliteConnection &conn, int64_t uid) {
        een9_ASSERT(uid >= 0, "Are you crazy?");
        SqliteStatement sql_req(conn,
           "SELECT `nickname`, `name` FROM `user` WHERE `id` = ?1",
           {{1, uid}}, {});
        fsql_text8_or_null nickname_col;
        fsql_text8_or_null name_col;
        int status = sqlite_stmt_step(sql_req, {}, {{0, &nickname_col}, {1, &name_col}});
        if (status == SQLITE_ROW) {
            return {std::move(nickname_col.value), std::move(name_col.value)};
        }
        return {};
    }

    RowChat_Content lookup_chat_content(SqliteConnection &conn, int64_t uid) {
        een9_ASSERT(uid >= 0, "Are you crazy?");
        SqliteStatement sql_req(conn,
           "SELECT `nickname`, `name`, `lastMsgId` FROM `chat` WHERE `id` = ?1",
           {{1, uid}}, {});
        fsql_text8_or_null nickname_col;
        fsql_text8_or_null name_col;
        fsql_integer_or_null last_msg_id_col;
        int status = sqlite_stmt_step(sql_req, {{2, &last_msg_id_col}}, {{0, &nickname_col}, {1, &name_col}});
        if (status == SQLITE_ROW) {
            return {std::move(nickname_col.value), std::move(name_col.value),
                last_msg_id_col.exist ? last_msg_id_col.value : -1};
        }
        return {};
    }

    void initial_extraction_of_all_the_useful_info_from_cookies(
        SqliteConnection& conn, const een9::ClientRequest& req,
        std::vector<std::pair<std::string, std::string>> &ret_cookies,
        std::vector<LoginCookie> &ret_login_cookies,
        json::JSON &ret_userinfo,
        int64_t& ret_logged_in_user
    ) {
        ret_cookies = een9::findAllClientCookies(req.headers);
        ret_login_cookies = select_login_cookies(ret_cookies);
        ret_logged_in_user = -1; /* Negative means that user is not logged in */
        if (!ret_login_cookies.empty()){
            size_t oldest_ind = select_oldest_login_cookie(ret_login_cookies);
            LoginCookie& tried = ret_login_cookies[oldest_ind];
            ret_logged_in_user = find_user_by_credentials(conn, tried.nickname, tried.password);
            if (ret_logged_in_user >= 0) {
                ret_userinfo["uid"] = json::JSON(ret_logged_in_user);
                ret_userinfo["nickname"] = json::JSON(tried.nickname);
                ret_userinfo["name"] = json::JSON(find_user_name(conn, ret_logged_in_user));
            }
        }
    }

    int64_t get_role_of_user_in_chat(SqliteConnection& conn, int64_t userId, int64_t chatId) {
        SqliteStatement req(conn,
            "SELECT `role` FROM `user_chat_membership` WHERE `userId` = ?1 AND `chatId` = ?2",
            {{1, userId}, {2, chatId}}, {});
        fsql_integer_or_null role;
        int status = sqlite_stmt_step(req, {{0, &role}}, {});
        if (status == SQLITE_ROW) {
            return role.exist ? (int)role.value : user_chat_role_deleted;
        }
        return user_chat_role_deleted;
    }

    std::string RTEE(const std::string& el_name,
                     const json::JSON& config_presentation, WorkerGuestData& wgd,
                     const json::JSON& userinfo) {
        std::string page = wgd.templater->render(el_name, {&config_presentation, &userinfo});
        return een9::form_http_server_response_200("text/html", page);
    }
}
